﻿using ProjectAiroplane.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectAiroplane.Drawnings;

public class Drawningplane
{
    /// <summary>
    /// Класс-сущность
    /// </summary>
    public Entityplane? Entityplane { get; protected set; }
    /// <summary>
    /// Ширина окна
    /// </summary>
    private int? _pictureWidth;
    /// <summary>
    /// Высота окна
    /// </summary>
    private int? _pictureHeight;
    /// <summary>
    /// Левая координата прорисовки 
    /// </summary>
    protected int? _startPosX;
    /// <summary>
    /// Верхняя кооридната прорисовки 
    /// </summary>
    protected int? _startPosY;
    /// <summary>
    /// Ширина прорисовки 
    /// </summary>
    private readonly int _drawningplaneWidth = 168;
    /// <summary>
    /// Высота прорисовки 
    /// </summary>
    private readonly int _drawningplaneHeight = 80;
    
    // <summary>
    /// Координата X объекта
    /// </summary>
    public int? GetPosX => _startPosX;

    /// <summary>
    /// Координата Y объекта
    /// </summary>
    public int? GetPosY => _startPosY;

    /// <summary>
    /// Ширина объекта
    /// </summary>
    public int GetWidth => _drawningplaneWidth;

    /// <summary>
    /// Высота объекта
    /// </summary>
    public int GetHeight => _drawningplaneHeight;

    /// <summary>
    /// Пустой конструктор
    /// </summary>
    private Drawningplane()
    {
        _pictureWidth = null;
        _pictureHeight = null;
        _startPosX = null;
        _startPosY = null;
    }
    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>

    public Drawningplane(int speed, double weight, Color bodyColor) : this()
    {
        Entityplane = new Entityplane(speed, weight, bodyColor);
    }

    /// <summary>
    /// Конструктор для наследников
    /// </summary>
    /// <param name="_drawningplaneWidth">Ширина прорисовки </param>
    /// <param name="_drawningplaneHeight">Высота прорисовки</param>

    protected Drawningplane(int _drawningplaneWidth, int _drawningplaneHeight) : this()
    {
        _pictureWidth = _drawningplaneWidth;/// tut posmotret
        _pictureHeight = _drawningplaneHeight;

        _pictureWidth = null;
        _pictureHeight = null;
        _startPosX = null;
        _startPosY = null;
    }

    /// <summary>
    /// Установка границ поля
    /// </summary>
    /// <param name="width">Ширина поля</param>
    /// <param name="height">Высота поля</param>
    /// <returns>true - границы заданы, false - проверка не пройдена, нельзя разместить объект в этих размерах</returns>
    public bool SetPictureSize(int width, int height)
    {
        // TODO проверка, что объект "влезает" в размеры поля
        // если влезает, сохраняем границы и корректируем позицию объекта,если она была уже установлена

        if (width >= _drawningplaneWidth && height >= _drawningplaneHeight)
        {
            _pictureWidth = width;
            _pictureHeight = height;

            if (_startPosX.HasValue && _startPosY.HasValue)
            {
                if (_startPosX + _drawningplaneWidth > _pictureWidth)
                {
                    _startPosX = _pictureWidth - _drawningplaneWidth;
                }

                if (_startPosY + _drawningplaneHeight > _pictureWidth)
                {
                    _startPosY = _pictureHeight - _drawningplaneHeight;
                }

            }
            return true;
        }
        return false;
    }
    /// <summary>
    /// Установка позиции
    /// </summary>
    /// <param name="x">Координата X</param>
    /// <param name="y">Координата Y</param>
    public void SetPosition(int x, int y)
    {
        if (!_pictureHeight.HasValue || !_pictureWidth.HasValue)
        {
            return;
        }
        // TODO если при установке объекта в эти координаты, он будет"выходить" за границы формы
        // то надо изменить координаты, чтобы он оставался в этих границах

        if (x < 0)
        {
            x = 0;
        }

        else if (x + _drawningplaneWidth > _pictureWidth.Value)
        {
            x = _pictureWidth.Value - _drawningplaneWidth;
        }

        if (y < 0)
        {
            y = 0;
        }

        else if (y + _drawningplaneHeight > _pictureHeight.Value)
        {
            y = _pictureHeight.Value - _drawningplaneHeight;
        }

        _startPosX = x;
        _startPosY = y;
    }

    /// <summary>
    /// Изменение направления перемещения
    /// </summary>
    /// <param name="direction">Направление</param>
    /// <returns>true - перемещене выполнено, false - перемещение невозможно</returns>
    public bool MoveTransport(DirectionType direction)
    {
        if (Entityplane == null || !_startPosX.HasValue ||
        !_startPosY.HasValue)
        {
            return false;
        }
        switch (direction)
        {
            //влево
            case DirectionType.Left:
                if (_startPosX.Value - Entityplane.Step > 0)
                {
                    _startPosX -= (int)Entityplane.Step;
                }
                return true;
            //вверх
            case DirectionType.Up:
                if (_startPosY.Value - Entityplane.Step > 0)
                {
                    _startPosY -= (int)Entityplane.Step;
                }
                return true;
            // вправо
            case DirectionType.Right:
                if (_startPosX.Value + _drawningplaneWidth + Entityplane.Step < _pictureWidth)
                {
                    _startPosX += (int)Entityplane.Step;
                }
                return true;
            //вниз
            case DirectionType.Down:
                if (_startPosY.Value + _drawningplaneHeight + Entityplane.Step < _pictureHeight)
                {
                    _startPosY += (int)Entityplane.Step;
                }
                return true;
            default:
                return false;
        }
    }
    /// <summary>
    /// Прорисовка объекта
    /// </summary>
    /// <param name="g"></param>
    public virtual void DrawTransport(Graphics g)
    {
        if (Entityplane == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return;
        }

        Pen pen = new(Color.Black);

        int f = 10;
        //крыло верхнее самолета
        g.DrawLine(pen, _startPosX.Value + 15 - f, _startPosY.Value + 5 - f, _startPosX.Value + 15 - f, _startPosY.Value + 5 - f);
        g.DrawLine(pen, _startPosX.Value + 15 - f, _startPosY.Value + 5 - f, _startPosX.Value + 55 - f, _startPosY.Value + 35 - f);
        g.DrawLine(pen, _startPosX.Value + 15 - f, _startPosY.Value + 35 - f, _startPosX.Value + 15 - f, _startPosY.Value + 5 - f);

        //задняя часть самолета полукруг
        g.DrawArc(pen, _startPosX.Value + 5 - f, _startPosY.Value + 35 - f, 30, 30, 90, 180);

        //сзади
        Brush bodybrush = new SolidBrush(Entityplane.BodyColor);
        g.DrawEllipse(pen, _startPosX.Value + 10 - f, _startPosY.Value + 30 - f, 40, 10);
        g.FillEllipse(bodybrush, _startPosX.Value + 10 - f, _startPosY.Value + 30 - f, 40, 10);
       
        //низ самолета
        g.DrawLine(pen, _startPosX.Value + 16 - f, _startPosY.Value + 65 - f, _startPosX.Value + 135 - f, _startPosY.Value + 65 - f);

        // треуголник спереди
        g.DrawLine(pen, _startPosX.Value + 135 - f, _startPosY.Value + 68 - f, _startPosX.Value + 135 - f, _startPosY.Value + 32 - f);
        g.DrawLine(pen, _startPosX.Value + 135 - f, _startPosY.Value + 32 - f, _startPosX.Value + 160 - f, _startPosY.Value + 50 - f);
        g.DrawLine(pen, _startPosX.Value + 135 - f, _startPosY.Value + 68 - f, _startPosX.Value + 160 - f, _startPosY.Value + 50 - f);
        g.DrawLine(pen, _startPosX.Value + 135 - f, _startPosY.Value + 50 - f, _startPosX.Value + 160 - f, _startPosY.Value + 50 - f);

        //вверх от тругольника до треугольника 
        g.DrawLine(pen, _startPosX.Value + 50 - f, _startPosY.Value + 35 - f, _startPosX.Value + 135 - f, _startPosY.Value + 35 - f);

        //крыло
        g.DrawEllipse(pen, _startPosX.Value + 50 - f, _startPosY.Value + 45 - f, 70, 10);
        g.FillEllipse(bodybrush, _startPosX.Value + 50 - f, _startPosY.Value + 45 - f, 70, 10);

        //шасси
        g.DrawLine(pen, _startPosX.Value + 35 - f, _startPosY.Value + 65 - f, _startPosX.Value + 35 - f, _startPosY.Value + 75 - f);
        g.DrawLine(pen, _startPosX.Value + 120 - f, _startPosY.Value + 65 - f, _startPosX.Value + 120 - f, _startPosY.Value + 75 - f);
        g.DrawLine(pen, _startPosX.Value + 25 - f, _startPosY.Value + 75 - f, _startPosX.Value + 45 - f, _startPosY.Value + 75 - f);
        g.DrawEllipse(pen, _startPosX.Value + 15 - f, _startPosY.Value + 70 - f, 10, 10);
        g.DrawEllipse(pen, _startPosX.Value + 45 - f, _startPosY.Value + 70 - f, 10, 10);
        g.DrawEllipse(pen, _startPosX.Value + 115 - f, _startPosY.Value + 70 - f, 10, 10);

    }
}
