﻿using ProjectAiroplane.Drawnings;
using ProjectAiroplane.MovementStrategy;

namespace ProjectAiroplane
{

    public partial class FormAiroplane : Form
    {
        private Drawningplane? _drawningplane;
        private AbstractStrategy? _strategy;
        public FormAiroplane()
        {
            InitializeComponent();
            _strategy = null;
        }

        private void Draw()
        {
            if (_drawningplane == null)
            {
                return;
            }
            Bitmap bmp = new(pictureBoxAiroplane.Width, pictureBoxAiroplane.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _drawningplane.DrawTransport(gr);
            pictureBoxAiroplane.Image = bmp;
        }

        private void CreateObject(string type)
        {
            Random random = new Random();
            switch (type)
            {
                case nameof(Drawningplane):
                    _drawningplane = new Drawningplane(random.Next(100, 300), random.Next(1000, 3000),
                Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)));
                    break;
                case nameof(DrawningAiroplane):
                    _drawningplane = new DrawningAiroplane(random.Next(100, 300), random.Next(1000, 3000),
                Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)),
                Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)),
                Convert.ToBoolean(random.Next(0, 2)), Convert.ToBoolean(random.Next(0, 2)));
                    break;
                default:
                    return;
            }
            _drawningplane.SetPictureSize(pictureBoxAiroplane.Width, pictureBoxAiroplane.Height);
            _drawningplane.SetPosition(random.Next(10, 100), random.Next(10, 100));
            _strategy = null;
            comboBoxStrategy.Enabled = true;
            Draw();
        }
        /// <summary>
        /// Кнопка создания самолета
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonCreate_Click(object sender, EventArgs e)
        {
            CreateObject(nameof(DrawningAiroplane));
        }

        ///// <summary>
        ///// Кнопка создания самолета с радаром "Создать самолет с радаром"
        ///// </summary>
        ///// <param name="sender"></param>
        ///// <param name="e"></param>
        //private void ButtonCreateAiroplane_Click(object sender, EventArgs e) => CreateObject(nameof(DrawningAiroplane));

        /// <summary>
        /// Кнопка создания самолета "Создать"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>

        private void ButtonCreateplane_Click(object sender, EventArgs e) => CreateObject(nameof(Drawningplane));

        /// <summary>
        /// Перемещение объекта по форме (нажатие кнопок навигации)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>

        private void ButtonMove_Click(object sender, EventArgs e)
        {
            if (_drawningplane == null)
            {
                return;
            }
            string name = ((Button)sender)?.Name ?? string.Empty;
            bool result = false;
            switch (name)
            {
                case "buttonUp":
                    result = _drawningplane.MoveTransport(DirectionType.Up);
                    break;
                case "buttonDown":
                    result = _drawningplane.MoveTransport(DirectionType.Down);
                    break;
                case "buttonLeft":
                    result = _drawningplane.MoveTransport(DirectionType.Left);
                    break;
                case "buttonRight":
                    result = _drawningplane.MoveTransport(DirectionType.Right);
                    break;
            }
            if (result)
            {
                Draw();
            }
        }

        private void ButtonStrategyStep_Click(object sender, EventArgs e)
        {
            if (_drawningplane == null)
            {
                return;
            }
            if (comboBoxStrategy.Enabled)
            {
                _strategy = comboBoxStrategy.SelectedIndex switch
                {
                    0 => new MoveToCenter(),
                    1 => new MoveToBorder(),
                    _ => null,
                };
                if (_strategy == null)
                {
                    return;
                }
                _strategy.SetData(new Moveableplane(_drawningplane),
                pictureBoxAiroplane.Width, pictureBoxAiroplane.Height);
            }
            if (_strategy == null)
            {
                return;
            }
            comboBoxStrategy.Enabled = false;
            _strategy.MakeStep();
            Draw();
            if (_strategy.GetStatus == StrategyStatus.Finish)
            {
                comboBoxStrategy.Enabled = true;
                _strategy = null;
            }
        }
    }
}

