﻿using ProjectAiroplane.Drawnings;

namespace ProjectAiroplane.MovementStrategy;

public class Moveableplane : IMoveableObjects
{
    private Drawningplane? _drawningplane;
    public Moveableplane(Drawningplane? drawningplane)
    {
        _drawningplane = drawningplane;
    }
    public ObjectParameters? GetObjectPosition
    {
        get
        {
            if (_drawningplane == null || _drawningplane.Entityplane == null || !_drawningplane.GetPosX.HasValue || !_drawningplane.GetPosY.HasValue)
            {
                return null;
            }
            return new ObjectParameters(_drawningplane.GetPosX.Value, _drawningplane.GetPosY.Value, _drawningplane.GetWidth, _drawningplane.GetHeight);
        }
    }
    public int GetStep => (int)(_drawningplane?.Entityplane?.Step ?? 0);
    public bool TryMoveObject(MovementDirection direction)
    {
        if (_drawningplane == null || _drawningplane.Entityplane == null)
        {
            return false;
        }
        return _drawningplane.MoveTransport(GetDirectionType(direction));
    }
    /// <summary>
    /// Конвертация из MovementDirection в DirectionType0
    /// </summary>
    /// <param name="direction">MovementDirection</param>
    /// <returns>DirectionType</returns>
    private static DirectionType GetDirectionType(MovementDirection direction)
    {
        return direction switch
        {
            MovementDirection.Left => DirectionType.Left,
            MovementDirection.Right => DirectionType.Right,
            MovementDirection.Up => DirectionType.Up,
            MovementDirection.Down => DirectionType.Down,
            _ => DirectionType.Unknow,
        };
    }
}